<?php

namespace App\Http\Controllers;

use App\Http\Controllers\Controller;
use App\Models\Log;
use App\Models\Device;
use Illuminate\Http\Request;
use App\Models\User;
use App\Models\Blogs;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class SiteController extends Controller
{
	public function dashboard()
	{
		return view('site/dashboard');
	}

	public function login(Request $request)
	{
		if ($request->cookie('user_remember_token')) {
			if (!$this->general->rateLimit('remember_login:' .$this->general->getClientIp())) {
				$userObj = new User();
				$result = $userObj->loginByAuthToken($request->cookie('user_remember_token'));
				if ($result['status']) {
					return redirect('dashboard');
				}
			}
		}
		return view('site/login');
	}


	public function loginProcess(Request $request)
	{
		if ($this->general->rateLimit('login:' . $this->general->getClientIp())) {
			return redirect()->back()->with('error', 'Too many attempts please try after some time.')->withInput();
		}
		$validator = Validator::make($request->all(), [
			'email' => 'required',
			'password' => 'required'
		]);
		if ($validator->fails()) {
			return redirect('login')->with('error', $this->general->getError($validator))->withInput();
		}
		$userObj = new User();
		$result = $userObj->loginProcess($request->all());
		if ($result['status']) {
			return redirect($result['user']->isAdmin()?'admin/dashboard':'dashboard')->with('success', $result['message']);
		} else {
			return redirect()->back()->with('error', $result['message'])->withInput();
		}
	}

	public function logout()
	{
		if(!Auth::guest()){
			Auth::logout();
			(new \App\Models\Device())->logout();
		}
		return redirect('admin/site/login')->withCookie(\Illuminate\Support\Facades\Cookie::forget('user_remember_token'));
	}

	public function register()
	{
		return view('site/register');
	}

	public function registerProcess(Request $request)
	{
		if ($this->general->rateLimit('register:' . $this->general->getClientIp())) {
			return redirect()->back()->with('error', 'Too many attempts please try after some time.')->withInput();
		}
		if($this->general->recaptchaFails()){
			return redirect()->back()->with('error', 'Please check recaptcha.')->withInput();
		}

		$validator = Validator::make($request->all(), [
			'first_name' => 'required|max:255',
			'last_name' => 'required|max:255',
			'email' =>  'required|email|unique:user',
			'password' => 'required',
			'password_confirm' => 'required_with:password|same:password',
		]);
		if ($validator->fails()) {
			return redirect('register')->with('error', $this->general->getError($validator))->withInput();
		}

		$userObj = new User();
		$result = $userObj->registerProcess($request->all(), $request->header('User-Agent'));
		return redirect('login')->with('success', $result['message']);
	}

	public function resendEmailVerification(Request $request)
	{
		if ($this->general->rateLimit('resend_email_verification:' . $this->general->getClientIp(), 3)) {
			return redirect()->back()->with('error', 'Too many attempts please try after some time.')->withInput();
		}

		$email = @base64_decode($request->input('code'));
		$user = User::where(['email' => $email])->first();
		if (!$user) {
			return redirect('login')->with('error', 'Email not found');
		}
		$user->email_verified = $user->generateToken();
		$user->save();
        
		$this->general->sendMail($user->email, 'Email Verify | ' . config('setting.app_name'), 'email/email_verify', [
			'name' => $user->first_name . ' ' . $user->last_name,
			'url' => route('email-verify', ['code' => $user->email_verified])
		]);
		return redirect('login')->with('success', 'Email Send Successfully');
	}

	public function emailVerify(Request $request)
	{
		if ($this->general->rateLimit('email_verity:' . $this->general->getClientIp())) {
			return redirect()->back()->with('error', 'Too many attempts please try after some time.')->withInput();
		}
		$code=$request->input('code');
		$user = User::where(['email_verified' => $code])->first();
		if (!$user) {
			return redirect('login')->with('error', 'Link is not valid');
		}
		if ($user->checkTokenExpired($code)) {
			return redirect('login')->with('error', 'Link is expired');
		}
		$user->email_verified = 1;
		$user->save();
		return redirect('login')->with('success', 'User verified Successfully');
	}

	public function passwordForgot()
	{
		return view('site/password_forgot');
	}

	public function passwordForgotProcess(Request $request)
	{
		if ($this->general->rateLimit('password_forgot:' . $this->general->getClientIp())) {
			return redirect()->back()->with('error', 'Too many attempts please try after some time.')->withInput();
		}

		if($this->general->recaptchaFails()){
			return redirect()->back()->with('error', 'Please check recaptcha.')->withInput();
		}

		$validator = Validator::make($request->all(), [
			'email' => 'required|email',
		]);
		if ($validator->fails()) {
			return redirect()->back()->with('error', $this->general->getError($validator))->withInput();
		}

		$user = User::where(['email' => $request->input('email')])->first();
		if (!$user) {
			return redirect()->back()->with('error', 'Email does not exist')->withInput();
		}
		$user->password_reset_token  = $user->generateToken();
		$user->save();
		$this->general->sendMail($user->email, 'Password Reset | ' . config('setting.app_name'), 'email/password_reset', [
			'name' => $user->first_name . ' ' . $user->last_name,
			'url' => route('site/password-reset', ['code' => $user->password_reset_token])
		]);

		return redirect()->back()->with('success', 'Check your email for reset password instructions')->withInput();
	}

	public function passwordReset(Request $request)
	{
		$code = $request->input('code');
		$user = User::where(['password_reset_token' => $code])->first();
		if (!$user) {
			return redirect('login')->with('error', 'Link is invalid');
		}
		if ($user->checkTokenExpired($code)) {
			return redirect('login')->with('error', 'Link is expired');
		}
		return view('site/password_reset', ['code' => $code]);
	}

	public function passwordResetProcess(Request $request)
	{
		if ($this->general->rateLimit('password_reset:' . $this->general->getClientIp())) {
			return redirect()->back()->with('error', 'Too many attempts please try after some time.')->withInput();
		}
		$validator = Validator::make($request->all(), [
			'code' => 'required',
			'password' => ['required', \Illuminate\Validation\Rules\Password::min(6)->letters()->mixedCase()->numbers()->symbols()->uncompromised(3)],
			'password_confirm' => 'required_with:password|same:password',
		]);
		if ($validator->fails()) {
			return redirect()->back()->with('error', $this->general->getError($validator))->withInput();
		}

		$code = $request->input('code');
		$user = User::where(['password_reset_token' => $code])->first();
		if (!$user) {
			return redirect()->back()->with('error', 'Token invalid')->withInput();
		}
		if ($user->checkTokenExpired($code)) {
			return redirect('login')->with('error', 'Link is expired');
		}
		$user->password = $user->encryptPassword($request->input('password'));
		$user->password_reset_token = '';
		$user->save();

		return redirect('login')->with('success', 'Reset password successfully');
	}

	/**Otp Login start */
	public function otpLogin()
	{
		return view('site/otp_login');
	}

	public function otpLoginProcess(Request $request)
	{
		$validator = Validator::make($request->all(), [
			'email' => 'required|email',
		]);
		if ($validator->fails()) {
			return redirect()->back()->with('error', $this->general->getError($validator))->withInput();
		}
		$user = User::where(['email' => $request['email']])->first();
		if (!$user) {
			return redirect()->back()->with('error', 'Email does not exist')->withInput();
		}
		if($user->status == 0){
            return redirect()->back()->with('error', 'Your account is not active')->withInput();
		}
		$user->sendLoginOtp($user);
		
		return redirect()->route('otp/check', ['code' => base64_encode($request['email'])])->with('success', 'Check your email for Login With otp')->withInput();
	}

	public function otpCheck(Request $request)
	{
		$code = $request->input('code');
		return view('site/otp_check', compact('code'));
	}

	public function otpProcess(Request $request)
	{
		$validator = Validator::make($request->all(), [
			'otp' => 'required|numeric|min:6',
			'code' => 'required',
		]);
		if ($validator->fails()) {
			return redirect()->back()->with('error', $this->general->getError($validator))->withInput();
		}
		$code = $request->input('code');
		$otp = $request->input('otp');
		$email = base64_decode($code);
		$user = User::where(['email' => $email])->first();
		if (!$user) {
			return redirect('login')->with('error', 'Link is invalid');
		}
		$result = $user->checkOtp($otp, $user->login_otp);
		if (!$result['status']) {
			return redirect()->back()->with('error', $result['message']);
		}
		Auth::login($user);

		$user->login_otp = '';
		$user->save();

		(new Activity())->add($user->id, 4);
		(new Device())->login($user->id);

		return redirect('dashboard')->with('success', 'Welcome');
	}
	/**Otp Login end */

	/**Social Login start*/
	public function scialLogin(Request $request)
    {
        return \Laravel\Socialite\Facades\Socialite::driver($request->route('type'))->redirect();
    }

    public function scialCallback(Request $request)
    {
        $socialType=$request->route('type');
        $socialUser = \Laravel\Socialite\Facades\Socialite::driver($socialType)->user();

        if($socialUser && $socialUser->getEmail()){
            
            $user = User::where(['email'=>$socialUser->getEmail()])->first();
            if ($user) {
                Auth::login($user);
                (new Activity())->add($user->id,5);
                (new Device())->login($user->id);
                return redirect(route('home'));
            } else {
                $name=$socialUser->getName();
                $name=explode(' ',$name);
                $firstName=@$name[0];
                $lastName=@$name[1];
                if(!$lastName){
                    $lastName=@isset($socialUser->user['family_name']) ? $socialUser->user['family_name'] : '';
                }
                $ip = $this->general->getClientIp();
                $ipInfo = $this->general->getIpInfo();
                $user=new User();
                $user = $user->create([
                    'first_name' => $firstName,
                    'last_name' => $lastName,
                    'email' => $socialUser->getEmail(),
                    'social_id' => $socialUser->id,
                    'social_type' => $this->getSocialType($socialType),
                    'password' => $user->encryptPassword($user->generateToken()),
                    'email_verified' => 1,
                    'registered_ip' => @$ip,
                    'country' => isset($ipInfo->country) ? $ipInfo->country : '',
                    'timezone' => config('app.timezone'),
                ]);
                (new Activity())->add($user->id,6);

                Auth::login($user);
                (new Activity())->add($user->id,5);
                (new Device())->login($user->id);
                return redirect('dashboard');
            }
        }
        
        return redirect('login')->withError('error','Login failed');
    }
    public function index() {
        $posts = Blogs::all();
        return response()->view('admin/sitemap/index', [
            'posts' => $posts
        ])->header('Content-Type', 'text/xml');
      }

	/**Social Login end*/

}
